/*******************************************************
 *
 *  File: armlinux.c
 *
 *  Author: Peter van Sebille (peter@yipton.demon.co.uk)
 *
 *  (c) Copyright 2001, Peter van Sebille
 *  All Rights Reserved
 *
 *******************************************************/

#include "boot2nd_if.h"
#include "lib.h"
#include "armlinux.h"


#define SIZE_IN_DWORDS(x) ( ((x) + 3) >> 2)


/***********************************************
 *
 * Stuff copied from the Linux kernel
 *
 ***********************************************/

/*
 * Linux kernel tags... (copied include/asm-arm/setup.h
 */
typedef unsigned 			u32;
typedef unsigned short		u16;
typedef unsigned char 		u8;
#include "setup.h"

#define MINORBITS   8
#define MINORMASK   ((1U << MINORBITS) - 1)
#define MAJOR(dev)  ((unsigned int) ((dev) >> MINORBITS))
#define MINOR(dev)  ((unsigned int) ((dev) & MINORMASK))
#define HASHDEV(dev)    ((unsigned int) (dev))
#define NODEV       0
#define MKDEV(ma,mi)    (((ma) << MINORBITS) | (mi))
#define B_FREE      0xffff      /* yuk */


/*******************************************
 *
 * Interface functions
 *
 *******************************************/

void armlinux_format_paramater_page(unsigned* param_page, const boot1st_param_t* boot1st_param, const unsigned* bootstring, int initrd_size)
{
	struct tag* 		tag;
	int					cmdlinesize;

	tag = (struct tag*) param_page;
	tag->hdr.size = SIZE_IN_DWORDS(sizeof(struct tag_core) + sizeof(struct tag_header));
	tag->hdr.tag = ATAG_CORE;
	tag->u.core.flags = 0;					/* 0 == readonly */
	tag->u.core.pagesize = PAGE_SIZE;
	tag->u.core.rootdev = MKDEV(1, 0);		/* 1== RAMDISK */
	tag = (struct tag*) (((u32*) tag) + tag->hdr.size);

	if (boot1st_param->initrd_reloc_dst && initrd_size)		
	{
    	tag->hdr.size = SIZE_IN_DWORDS(sizeof(struct tag_initrd) + sizeof(struct tag_header));
    	tag->hdr.tag = ATAG_INITRD;
    	tag->u.initrd.start = (u32) boot1st_param->initrd_reloc_dst;
    	tag->u.initrd.size = (u32) initrd_size;
    	tag = (struct tag*) (((u32*) tag) + tag->hdr.size);
	}
		
	cmdlinesize = strlen((const char*) bootstring);
	if (cmdlinesize)
	{
    	tag->hdr.size = SIZE_IN_DWORDS(cmdlinesize + sizeof(struct tag_header));
    	tag->hdr.tag = ATAG_CMDLINE;
    	strcpy(tag->u.cmdline.cmdline, (const char*) bootstring);
    	tag->u.cmdline.cmdline[cmdlinesize] = 0;
    	tag = (struct tag*) (((u32*) tag) + tag->hdr.size);
	}
	
	tag->hdr.size = 0;
	tag->hdr.tag = 0;
}




